import textwrap


def parse_script_args(args):
	"""
	Separate the command line arguments into arguments for pip
	and arguments to Python.

	>>> parse_script_args(['foo', '--', 'bar'])
	(['foo'], ['bar'])

	>>> parse_script_args(['foo', 'bar'])
	(['foo', 'bar'], [])
	"""
	try:
		pivot = args.index('--')
	except ValueError:
		pivot = len(args)
	return args[:pivot], args[pivot+1:]


help_doc = textwrap.dedent("""
	Usage:

	Arguments to rwt prior to `--` are used to specify the requirements
	to make available, just as arguments to pip install. For example,

	    rwt -r requirements.txt "requests>=2.0"

	That will launch python after installing the deps in requirements.txt
	and also a late requests. Packages are always installed to a temporary
	location and cleaned up when the process exits.

	Arguments after `--` are passed to the Python interpreter. So to launch
	`script.py`:

	    rwt -- script.py

	If the `--` is ommitted or nothing is passed, the python interpreter
	will be launched in interactive mode:

	    rwt
	    >>>

	For more examples and details, see https://pypi.org/project/rwt.
	""").lstrip()


def intercept(args):
	"""
	Detect certain args and intercept them.
	"""
	if '--help' in args or '-h' in args:
		print(help_doc)
		raise SystemExit(0)



from __future__ import unicode_literals

import sys
import subprocess
import textwrap
import os

import pytest

from rwt import launch


def test_with_path(tmpdir, capfd):
	params = ['-c', 'import sys; sys.stdout.write("\\n".join(sys.path))']
	res = launch.with_path(str(tmpdir), params)
	assert res == 0
	out, err = capfd.readouterr()
	assert tmpdir in out.split(os.linesep)


def test_with_path_result_code(tmpdir):
	"""
	result code should be non-zero on error
	"""
	params = ['-c', "raise ValueError()"]
	res = launch.with_path(str(tmpdir), params)
	assert res > 0


@pytest.mark.xfail(reason="cleanup can't occur with execv; #4")
def test_with_path_overlay(tmpdir, capfd):
	params = ['-c', 'import sys; sys.stdout.write("\\n".join(sys.path))']
	# launch subprocess so as not to overlay the test process
	script = textwrap.dedent("""
		import rwt.launch
		rwt.launch.with_path_overlay({tmpdir!r}, {params!r})
		print("cleanup")
	""").strip().replace('\n', '; ').format(tmpdir=str(tmpdir), params=params)
	subprocess.Popen([sys.executable, '-c', script]).wait()
	out, err = capfd.readouterr()
	assert str(tmpdir) in out.split(os.linesep)
	assert "cleanup" in out


@pytest.fixture
def clean_pythonpath(monkeypatch):
	monkeypatch.delitem(os.environ, 'PYTHONPATH', raising=False)


def test_build_env(clean_pythonpath):
	os.environ['PYTHONPATH'] = 'something'
	env = launch._build_env('else')
	expected = os.pathsep.join(('else', 'something'))
	assert env['PYTHONPATH'] == expected

	os.environ['PYTHONPATH'] = ''
	env = launch._build_env('something')
	assert env['PYTHONPATH'] == 'something'

	initial = os.pathsep.join(['something', 'else'])
	os.environ['PYTHONPATH'] = initial
	env = launch._build_env('a')
	expected = os.pathsep.join(['a', 'something', 'else'])
	assert env['PYTHONPATH'] == expected


def test_build_env_includes_pth_files(tmpdir, clean_pythonpath):
	"""
	If during _build_env, there are .pth files in the target directory,
	they should be processed to include any paths indicated there.
	See #6 for rationale.
	"""
	(tmpdir / 'foo.pth').write_text('pkg-1.0', encoding='utf-8')
	env = launch._build_env(str(tmpdir))
	expected = os.pathsep.join([str(tmpdir), str(tmpdir / 'pkg-1.0')])
	assert env['PYTHONPATH'] == expected



from __future__ import unicode_literals

import os
import textwrap
import sys
import subprocess

from rwt import scripts


def test_pkg_imported(tmpdir):
	"""
	Create a script that loads cython and ensure it runs.
	"""
	body = textwrap.dedent("""
		import path
		print("Successfully imported path.py")
		""").lstrip()
	script_file = tmpdir / 'script'
	script_file.write_text(body, 'utf-8')
	pip_args = ['path.py']
	cmd = [sys.executable, '-m', 'rwt'] + pip_args + ['--', str(script_file)]

	out = subprocess.check_output(cmd, universal_newlines=True)
	assert 'Successfully imported path.py' in out


class TestDepsReader:
	def test_reads_files_with_attribute_assignment(self):
		script = textwrap.dedent('''
			__requires__=['foo']
			x.a = 'bar'
			''')
		assert scripts.DepsReader(script).read() == ['foo']

	def test_reads_files_with_multiple_assignment(self):
		script = textwrap.dedent('''
			__requires__=['foo']
			x, a = [a, x]
			''')
		assert scripts.DepsReader(script).read() == ['foo']

	def test_single_dep(self):
		script = textwrap.dedent('''
			__requires__='foo'
			''')
		assert scripts.DepsReader(script).read() == ['foo']

	def test_index_url(self):
		script = textwrap.dedent('''
			__requires__ = ['foo']
			__index_url__ = 'https://my.private.index/'
			''')
		reqs = scripts.DepsReader(script).read()
		assert reqs.index_url == 'https://my.private.index/'

	def test_dependency_links(self):
		script = textwrap.dedent('''
			__requires__ = ['foo==0.42']
			__dependency_links__ = ['git+ssh://git@example.com/repo.git#egg=foo-0.42']
			''')
		reqs = scripts.DepsReader(script).read()
		assert reqs.dependency_links == ['git+ssh://git@example.com/repo.git#egg=foo-0.42']


def test_pkg_loaded_from_alternate_index(tmpdir):
	"""
	Create a script that loads cython from an alternate index
	and ensure it runs.
	"""
	body = textwrap.dedent("""
		__requires__ = ['path.py']
		__index_url__ = 'https://devpi.net/root/pypi/+simple/'
		import path
		print("Successfully imported path.py")
		""").lstrip()
	script_file = tmpdir / 'script'
	script_file.write_text(body, 'utf-8')
	cmd = [sys.executable, '-m', 'rwt', '--', str(script_file)]

	out = subprocess.check_output(cmd, universal_newlines=True)
	assert 'Successfully imported path.py' in out
	assert 'devpi.net' in out


def test_pkg_loaded_from_dependency_links(tmpdir):
	"""
	Create a script whose dependency is only installable
	from a custom dependency link and ensure it runs.
	"""
	dependency = tmpdir.ensure_dir('barbazquux-1.0')
	(dependency / 'setup.py').write_text(textwrap.dedent(
		'''
		from setuptools import setup
		setup(
			name='barbazquux', version='1.0',
			py_modules=['barbazquux'],
		)
		'''
	), 'utf-8')
	(dependency / 'barbazquux.py').write_text('', 'utf-8')
	dependency_link = 'file://%s#egg=barbazquux-1.0' % (
		dependency.strpath.replace(os.path.sep, '/'),
	)
	body = textwrap.dedent("""
		__requires__ = ['barbazquux']
		__dependency_links__ = [{dependency_link!r}]
		import barbazquux
		print("Successfully imported barbazquux.py")
		""").lstrip().format(
			dependency_link=dependency_link
		)
	script_file = tmpdir.ensure_dir('script_dir') / 'script'
	script_file.write_text(body, 'utf-8')
	cmd = [sys.executable, '-m', 'rwt', '--no-index', '--', str(script_file)]
	out = subprocess.check_output(cmd, universal_newlines=True)
	assert 'Successfully imported barbazquux.py' in out
